; TIATracker music player (MADS format)
; Copyright 2016 Andre "Kylearan" Wichmann
; Website: https://bitbucket.org/kylearan/tiatracker
; Email: andre.wichmann@gmx.de
;
; Licensed under the Apache License, Version 2.0 (the "License");
; you may not use this file except in compliance with the License.
; You may obtain a copy of the License at
;
;   http://www.apache.org/licenses/LICENSE-2.0
;
; Unless required by applicable law or agreed to in writing, software
; distributed under the License is distributed on an "AS IS" BASIS,
; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
; See the License for the specific language governing permissions and
; limitations under the License.

; Song author: Glafouk
; Song name: Snake It (SV Summer 2021)

; @com.wudsn.ide.asm.hardware=ATARI2600

        icl "VCS.asm"
        
; TV format switches
PAL             = 1
NTSC            = 0

        .if PAL
TIM_VBLANK      = 43
TIM_OVERSCAN    = 36
TIM_KERNEL      = 19
        .else
TIM_VBLANK      = 45
TIM_OVERSCAN    = 38
TIM_KERNEL      = 15
        .endif


; =====================================================================
; Variables
; =====================================================================

        opt h-f-
        org     $80

        icl "SnakeIt_variables.asm"

; test
startup_delay	.ds 1
player_time_max         .ds 1


; =====================================================================
; Start of code
; =====================================================================

        opt f+
        org     $f000

        .proc Start
	jmp skip
	.byte 10,13,0
	.byte 'SnakeIt by Glafouk! for SillyVenture 2k20 Summer Edition',10,13,0
	.byte 'Info screen by JAC! on 2022-08-13',10,13,0

        ; Clear zeropage        
skip
        cld
        ldx #0
        txa
clearLoop:
        dex
        txs
        pha
        bne clearLoop

        icl "SnakeIt_init.asm"
	mva #150 startup_delay
        
; =====================================================================
; MAIN LOOP
; =====================================================================

MainLoop:

	lda swchb		;RESET (F2) pressed?
	and #1
	jeq start

; ---------------------------------------------------------------------
; Overscan
; ---------------------------------------------------------------------

        .proc Overscan       

        sta WSYNC
        lda #2
        sta VBLANK
        lda #TIM_OVERSCAN
        sta TIM64T

        ; Do overscan stuff

:10	nop
	sta resp0
	sta resp1
	mva #70 hmp1
	sta wsync
	sta hmove
	sta wsync
	sta hmove
	lda #6
	sta nusiz0
	sta nusiz1

waitForIntim
        lda INTIM
        bne waitForIntim
        .endp

; ---------------------------------------------------------------------
; VBlank
; ---------------------------------------------------------------------

        .proc VBlank

        lda #%1110
vsyncLoop:
        sta WSYNC
        sta VSYNC
        lsr
        bne vsyncLoop
        lda #2
        sta VBLANK
        lda #TIM_VBLANK
        sta TIM64T

	lda startup_delay
	beq play
	dec startup_delay
	jne dont_play
	; Do VBlank stuff
play	icl "SnakeIt_player.asm"
dont_play
        
        ; Measure player worst case timing
        lda #TIM_VBLANK
        sec
        sbc INTIM
        cmp player_time_max
        bcc noNewMax
        sta player_time_max
noNewMax:

waitForVBlank:
        lda INTIM
        bne waitForVBlank
        sta WSYNC
        sta VBLANK
        .endp

; ---------------------------------------------------------------------
; Kernel
; ---------------------------------------------------------------------

        .proc Kernel
        lda #TIM_KERNEL
        sta T1024T

        ; Do kernel stuff
	ldx #0
center	stx wsync
	inx
	cpx #8
	bne center

	ldx #0
upper	stx wsync
	stx colubk
	inx
	cpx #16
	bne upper

	ldx #0
effect	lda colors,x
	stx wsync
	sta colup0
	lda gfx,x
	sta grp0
	lda gfx-1,x
	sta grp1
	inx
	cpx #192
	bne effect

	ldx #15
lower	stx wsync
	stx colubk
	dex
	bpl lower


waitForIntim:
        lda INTIM
        bne waitForIntim
        .endp

        jmp MainLoop
        .endp

; =====================================================================
; Data
; =====================================================================

colors
	red = $60
	olive = $40
	yellow = $20
	violet = $a0
	blue_violet = $c0
	blue_seven = $b0
	blue = $d0
	blue_green = $90
	green = $50
	gold = $20

	.macro m_col
	.byte :1+$00
	.byte :1+$02
	.byte :1+$04
	.byte :1+$06
	.byte :1+$08
	.byte :1+$0a
	.byte :1+$0c
	.byte :1+$0e
	.endm

	m_col $00
	m_col $00
	m_col red
	m_col yellow
	m_col olive	
	m_col green
	m_col blue_green
	m_col blue
	m_col blue_violet
	m_col violet
	m_col $00
	m_col $00
	m_col $00
	m_col $00
	m_col red
	m_col yellow
	m_col olive	
	m_col green
	m_col blue_green
	m_col blue
	m_col blue_violet
	m_col violet

	.byte 0
gfx	ins "gfx/Snakeit.pic"

        icl "SnakeIt_trackdata.asm"

; =====================================================================
; Vectors
; =====================================================================

        .echo "ROM left: ", ($fffc - *)

        org $fffc
        .word   Start
        .word   Start
