; TIATracker music player  (MADS format)
; Copyright 2016 Andre "Kylearan" Wichmann
; Website: https://bitbucket.org/kylearan/tiatracker
; Email: andre.wichmann@gmx.de
;
; Licensed under the Apache License, Version 2.0 (the "License");
; you may not use this file except in compliance with the License.
; You may obtain a copy of the License at
;
;   http://www.apache.org/licenses/LICENSE-2.0
;
; Unless required by applicable law or agreed to in writing, software
; distributed under the License is distributed on an "AS IS" BASIS,
; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
; See the License for the specific language governing permissions and
; limitations under the License.

; Song author: Glafouk
; Song name: Snake It (SV Summer 2021)

; @com.wudsn.ide.asm.hardware=ATARI2600

; =====================================================================
; TIATracker melodic and percussion instruments, patterns and sequencer
; data.
; =====================================================================
tt_TrackDataStart:

; =====================================================================
; Melodic instrument definitions (up to 7). tt_envelope_index_c0/1 hold
; the index values into these tables for the current instruments played
; in channel 0 and 1.
; 
; Each instrument is defined by:
; - tt_InsCtrlTable: the AUDC value
; - tt_InsADIndexes: the index of the start of the ADSR envelope as
;       defined in tt_InsFreqVolTable
; - tt_InsSustainIndexes: the index of the start of the Sustain phase
;       of the envelope
; - tt_InsReleaseIndexes: the index of the start of the Release phase
; - tt_InsFreqVolTable: The AUDF frequency and AUDV volume values of
;       the envelope
; =====================================================================

; Instrument master CTRL values
tt_InsCtrlTable:
        .byte $06, $04, $0c


; Instrument Attack/Decay start indexes into ADSR tables.
tt_InsADIndexes:
        .byte $00, $0a, $0a


; Instrument Sustain start indexes into ADSR tables
tt_InsSustainIndexes:
        .byte $06, $0c, $0c


; Instrument Release start indexes into ADSR tables
; Caution: Values are stored with an implicit -1 modifier! To get the
; real index, add 1.
tt_InsReleaseIndexes:
        .byte $07, $0d, $0d


; AUDVx and AUDFx ADSR envelope values.
; Each byte encodes the frequency and volume:
; - Bits 7..4: Freqency modifier for the current note ([-8..7]),
;       8 means no change. Bit 7 is the sign bit.
; - Bits 3..0: Volume
; Between sustain and release is one byte that is not used and
; can be any value.
; The end of the release phase is encoded by a 0.
tt_InsFreqVolTable:
; 0: bassline
        .byte $8f, $8f, $8d, $8b, $89, $87, $85, $00
        .byte $83, $00
; 1+2: Chords
        .byte $8d, $8d, $83, $00, $80, $00



; =====================================================================
; Percussion instrument definitions (up to 15)
;
; Each percussion instrument is defined by:
; - tt_PercIndexes: The index of the first percussion frame as defined
;       in tt_PercFreqTable and tt_PercCtrlVolTable
; - tt_PercFreqTable: The AUDF frequency value
; - tt_PercCtrlVolTable: The AUDV volume and AUDC values
; =====================================================================

; Indexes into percussion definitions signifying the first frame for
; each percussion in tt_PercFreqTable.
; Caution: Values are stored with an implicit +1 modifier! To get the
; real index, subtract 1.
tt_PercIndexes:
        .byte $01


; The AUDF frequency values for the percussion instruments.
; If the second to last value is negative (>=128), it means it's an
; "overlay" percussion, i.e. the player fetches the next instrument note
; immediately and starts it in the sustain phase next frame. (Needs
; TT_USE_OVERLAY)
tt_PercFreqTable:
; 0: Snare
        .byte $05, $1b, $1b, $02, $08, $07, $00


; The AUDCx and AUDVx volume values for the percussion instruments.
; - Bits 7..4: AUDC value
; - Bits 3..0: AUDV value
; 0 means end of percussion data.
tt_PercCtrlVolTable:
; 0: Snare
        .byte $8f, $cf, $ef, $8d, $8a, $83, $00


        
; =====================================================================
; Track definition
; The track is defined by:
; - tt_PatternX (X=0, 1, ...): Pattern definitions
; - tt_PatternPtrLo/Hi: Pointers to the tt_PatternX tables, serving
;       as index values
; - tt_SequenceTable: The order in which the patterns should be played,
;       i.e. indexes into tt_PatternPtrLo/Hi. Contains the sequences
;       for all channels and sub-tracks. The variables
;       tt_cur_pat_index_c0/1 hold an index into tt_SequenceTable for
;       each channel.
;
; So tt_SequenceTable holds indexes into tt_PatternPtrLo/Hi, which
; in turn point to pattern definitions (tt_PatternX) in which the notes
; to play are specified.
; =====================================================================

; ---------------------------------------------------------------------
; Pattern definitions, one table per pattern. tt_cur_note_index_c0/1
; hold the index values into these tables for the current pattern
; played in channel 0 and 1.
;
; A pattern is a sequence of notes (one byte per note) ending with a 0.
; A note can be either:
; - Pause: Put melodic instrument into release. Must only follow a
;       melodic instrument.
; - Hold: Continue to play last note (or silence). Default "empty" note.
; - Slide (needs TT_USE_SLIDE): Adjust frequency of last melodic note
;       by -7..+7 and keep playing it
; - Play new note with melodic instrument
; - Play new note with percussion instrument
; - End of pattern
;
; A note is defined by:
; - Bits 7..5: 1-7 means play melodic instrument 1-7 with a new note
;       and frequency in bits 4..0. If bits 7..5 are 0, bits 4..0 are
;       defined as:
;       - 0: End of pattern
;       - [1..15]: Slide -7..+7 (needs TT_USE_SLIDE)
;       - 8: Hold
;       - 16: Pause
;       - [17..31]: Play percussion instrument 1..15
;
; The tracker must ensure that a pause only follows a melodic
; instrument or a hold/slide.
; ---------------------------------------------------------------------
TT_FREQ_MASK    = %00011111
TT_INS_HOLD     = 8
TT_INS_PAUSE    = 16
TT_FIRST_PERC   = 17

; b0a
tt_pattern0:
        .byte $35, $08, $35, $08, $35, $35, $08, $31
        .byte $08, $31, $08, $31, $31, $31, $31, $31
        .byte $2b, $08, $2b, $08, $2b, $2b, $08, $2f
        .byte $08, $2f, $08, $2f, $2f, $2f, $2d, $2f
        .byte $00

; b0b
tt_pattern1:
        .byte $35, $08, $35, $08, $35, $35, $08, $31
        .byte $08, $31, $08, $31, $31, $31, $31, $31
        .byte $2f, $08, $2f, $08, $2f, $2f, $08, $2f
        .byte $08, $2f, $08, $2f, $2f, $2f, $2d, $2f
        .byte $00

; b0c
tt_pattern2:
        .byte $35, $08, $35, $08, $35, $35, $08, $31
        .byte $08, $31, $08, $31, $31, $31, $31, $31
        .byte $2f, $08, $2f, $08, $2f, $2f, $08, $2f
        .byte $08, $2f, $08, $2f, $2f, $2f, $31, $32
        .byte $00

; b0a+mel0a
tt_pattern3:
        .byte $35, $7b, $35, $77, $35, $35, $7f, $31
        .byte $7b, $31, $7f, $31, $31, $31, $31, $31
        .byte $2b, $74, $2b, $77, $2b, $2b, $7f, $2f
        .byte $7b, $2f, $77, $2f, $2f, $2f, $2d, $2f
        .byte $00

; b0b+mel0a
tt_pattern4:
        .byte $35, $7b, $35, $77, $35, $35, $74, $31
        .byte $72, $31, $7b, $31, $31, $31, $31, $31
        .byte $2f, $77, $2f, $74, $2f, $2f, $7f, $2f
        .byte $7b, $2f, $77, $2f, $2f, $2f, $2d, $2f
        .byte $00

; b0c+mel0a
tt_pattern5:
        .byte $35, $74, $35, $77, $35, $35, $7b, $31
        .byte $77, $31, $7f, $31, $31, $31, $31, $31
        .byte $2f, $7b, $2f, $77, $2f, $2f, $7f, $2f
        .byte $7b, $2f, $7b, $2f, $2f, $2f, $31, $32
        .byte $00

; endB
tt_pattern6:
        .byte $35, $08, $08, $08, $08, $08, $10, $08
        .byte $08, $08, $08, $08, $08, $08, $08, $08
        .byte $00

; endC
tt_pattern7:
        .byte $08, $08, $08, $08, $08, $08, $08, $08
        .byte $08, $08, $08, $08, $08, $08, $08, $08
        .byte $00

; d0
tt_pattern8:
        .byte $7b, $7b, $11, $08, $7b, $08, $11, $08
        .byte $7b, $7f, $11, $08, $7b, $77, $11, $08
        .byte $7b, $7b, $11, $08, $7b, $08, $11, $08
        .byte $77, $7f, $11, $08, $7b, $11, $11, $11
        .byte $00

; d+mel0a
tt_pattern9:
        .byte $6d, $6d, $11, $54, $6f, $6d, $11, $6b
        .byte $57, $6f, $11, $6d, $6d, $6f, $11, $6f
        .byte $6d, $6d, $11, $4f, $5f, $5f, $11, $6b
        .byte $51, $6d, $11, $6f, $6d, $11, $11, $11
        .byte $00

; d+mel0b
tt_pattern10:
        .byte $6d, $6d, $11, $54, $6f, $72, $11, $6f
        .byte $57, $6d, $11, $77, $74, $72, $11, $74
        .byte $7b, $4f, $11, $7b, $7f, $7b, $11, $77
        .byte $4f, $7b, $11, $54, $57, $11, $11, $11
        .byte $00

; d+mel0c
tt_pattern11:
        .byte $72, $72, $11, $6f, $54, $72, $11, $6f
        .byte $6d, $6b, $11, $72, $57, $6f, $11, $54
        .byte $6d, $6d, $11, $6f, $5b, $72, $11, $72
        .byte $74, $77, $11, $7f, $7b, $11, $11, $11
        .byte $00

; d+mel1a
tt_pattern12:
        .byte $5b, $5b, $11, $57, $4d, $5b, $11, $5f
        .byte $5b, $4f, $11, $6d, $6b, $5f, $11, $54
        .byte $5b, $5b, $11, $5f, $51, $6d, $11, $4f
        .byte $6b, $6b, $11, $6d, $54, $11, $11, $11
        .byte $00

; d+mel1b
tt_pattern13:
        .byte $54, $54, $11, $51, $4b, $5b, $11, $4f
        .byte $51, $4d, $11, $54, $57, $54, $11, $54
        .byte $4d, $4d, $11, $4f, $4b, $54, $11, $57
        .byte $54, $51, $11, $4f, $54, $11, $11, $11
        .byte $00

; d+mel1c
tt_pattern14:
        .byte $51, $51, $11, $54, $4d, $57, $11, $54
        .byte $51, $4f, $11, $54, $4f, $51, $11, $4b
        .byte $5b, $5b, $11, $57, $4d, $5b, $11, $6b
        .byte $6d, $6f, $11, $6d, $6b, $11, $11, $11
        .byte $00

; d+mel2a
tt_pattern15:
        .byte $6d, $5b, $11, $57, $5b, $4d, $11, $5f
        .byte $4f, $6d, $11, $6d, $6b, $6f, $11, $6d
        .byte $54, $6b, $11, $6f, $51, $6d, $11, $5b
        .byte $5f, $6d, $11, $5b, $57, $11, $11, $11
        .byte $00

; d+mel2b
tt_pattern16:
        .byte $6d, $5b, $11, $5b, $54, $4f, $11, $54
        .byte $57, $54, $11, $6d, $6b, $5f, $11, $6d
        .byte $54, $6b, $11, $4f, $54, $57, $11, $5b
        .byte $5f, $6d, $11, $4d, $6d, $11, $11, $11
        .byte $00

; d+mel2c
tt_pattern17:
        .byte $57, $5b, $11, $5f, $6d, $4f, $11, $4f
        .byte $4d, $4b, $11, $4f, $54, $57, $11, $6d
        .byte $54, $6b, $11, $6f, $51, $6d, $11, $5b
        .byte $5f, $6d, $11, $5b, $57, $11, $11, $11
        .byte $00

; mel1a
tt_pattern18:
        .byte $5b, $5b, $54, $57, $4d, $5b, $6d, $5f
        .byte $5b, $4f, $5b, $6d, $6b, $5f, $4d, $54
        .byte $5b, $5b, $4b, $5f, $51, $6d, $48, $4f
        .byte $6b, $6b, $4d, $6d, $54, $11, $11, $11
        .byte $00

; endA
tt_pattern19:
        .byte $08, $08, $11, $08, $08, $08, $11, $08
        .byte $08, $08, $11, $08, $08, $11, $11, $11
        .byte $00




; Individual pattern speeds (needs TT_GLOBAL_SPEED = 0).
; Each byte encodes the speed of one pattern in the order
; of the tt_PatternPtr tables below.
; If TT_USE_FUNKTEMPO is 1, then the low nibble encodes
; the even speed and the high nibble the odd speed.
    .if TT_GLOBAL_SPEED = 0
tt_PatternSpeeds:

    .endif


; ---------------------------------------------------------------------
; Pattern pointers look-up table.
; ---------------------------------------------------------------------
tt_PatternPtrLo:
        .byte <tt_pattern0, <tt_pattern1, <tt_pattern2, <tt_pattern3
        .byte <tt_pattern4, <tt_pattern5, <tt_pattern6, <tt_pattern7
        .byte <tt_pattern8, <tt_pattern9, <tt_pattern10, <tt_pattern11
        .byte <tt_pattern12, <tt_pattern13, <tt_pattern14, <tt_pattern15
        .byte <tt_pattern16, <tt_pattern17, <tt_pattern18, <tt_pattern19

tt_PatternPtrHi:
        .byte >tt_pattern0, >tt_pattern1, >tt_pattern2, >tt_pattern3
        .byte >tt_pattern4, >tt_pattern5, >tt_pattern6, >tt_pattern7
        .byte >tt_pattern8, >tt_pattern9, >tt_pattern10, >tt_pattern11
        .byte >tt_pattern12, >tt_pattern13, >tt_pattern14, >tt_pattern15
        .byte >tt_pattern16, >tt_pattern17, >tt_pattern18, >tt_pattern19
        


; ---------------------------------------------------------------------
; Pattern sequence table. Each byte is an index into the
; tt_PatternPtrLo/Hi tables where the pointers to the pattern
; definitions can be found. When a pattern has been played completely,
; the next byte from this table is used to get the address of the next
; pattern to play. tt_cur_pat_index_c0/1 hold the current index values
; into this table for channels 0 and 1.
; If TT_USE_GOTO is used, a value >=128 denotes a goto to the pattern
; number encoded in bits 6..0 (i.e. value AND %01111111).
; ---------------------------------------------------------------------
tt_SequenceTable:
        ; ---------- Channel 0 ----------
        .byte $00, $01, $00, $02, $03, $04, $03, $05
        .byte $03, $04, $03, $05, $03, $04, $03, $05
        .byte $03, $04, $03, $05, $00, $02, $06, $07
        .byte $96

        
        ; ---------- Channel 1 ----------
        .byte $08, $08, $08, $08, $09, $0a, $09, $0b
        .byte $0c, $0d, $0c, $0e, $0f, $10, $0f, $11
        .byte $0c, $0d, $12, $0e, $13, $13, $07, $07
        .byte $07, $07, $b1


        .echo "Track size: ", *-tt_TrackDataStart
